package Write;

my $pkg="Write";

use strict;
use My;

#== napojeni na db
my $dbh=My->dbh;


#- zapise metodou insert zmeny novy radek uvedeny v hashi %$zmena (pokud nejake polozky nejsou uvedeny bude zapsana jejich default hodnota - u pocitadel se hodnota navysi)
#  (obsahem %$zmena nemusi byt i polozky klice, ale mohou a nemusim rozlisovat klice prixem _key_, ale mohu (vhodne pro kompatibilitu hashe %$zmena u replace))
# priklad
# Write->insert({
#    _table => "Vyroba_evidence.Prijemvyr",
#    org => get_env("org"),
#    ....
# });
sub insert {
   my ($pkg_p, $zmena,$metoda)=@_;
   # metodu zapisu je mozne zmenit - muze byt i replace, ale defaultne jde o insert
   if(!defined($metoda)) {
      $metoda="insert";
   }
   # zmena datzapis a uzivzapis, ktere predpokladam u kazde tabulky
   if(!defined($$zmena{datzapis}) and !$$zmena{_bezdatzapis}) { # jen kdyz neni definovano jiz v $zmena
      $$zmena{datzapis}=My->datetime_now;
      $$zmena{uzivzapis}=My->remote_user;
   }
   # kolony co se budou zapisovat - vse co je v $zmena
   my $insert=q{
      } . $metoda . q{
         } . $$zmena{_table} . q{
      set
   };
   my @values;
   foreach my $name (sort keys %{$zmena}) {
      if($name=~/^([^_].*)$/ or $name=~/^_key_(.+)$/ or $name=~/^_add_(.+)$/) {
         $insert.= $1 . "=" . "?,";
         push(@values,$$zmena{$name});
      }
   }
   chop($insert); # odstraneni carky na konci
   my $sth=$dbh->prepare($insert);
   $sth->execute(@values);
}

#- zapise metodou replace (ale pokud zaznam existuje tak metodou update) zmeny uvedene v hashi %$zmena
#  (obsahem %$zmena musi byt i polozky klice), pokud jiz existuje
#  zaznam k zadanemu klici bude zmeneno jen to co je definovano v hashi %$zmena (pokud obsahuje vse - zmeni se vse)
# priklad jak pridat 5 ks k ksprij v tabulce Vyroba_evidence.Stavvyr:
# Write->replace({
#    _table => "Vyroba_evidence.Stavvyr",
#    _key_org => get_env("org"),
#    _key_zak => "XXXXXXXXXXXXXX",
#    _key_kmint => "XXX",
#    _key_str => "XXXX",
#    _add_ksprij => 5,
#    datprij => My->date_now
# });
sub replace {
   my ($pkg_p, $zmena)=@_;
   # zmena datzapis a uzivzapis, ktere predpokladam u kazde tabulky
   if(!defined($$zmena{datzapis}) and !$$zmena{_bezdatzapis}) { # jen kdyz neni definovano jiz v $zmena
      $$zmena{datzapis}=My->datetime_now;
      $$zmena{uzivzapis}=My->remote_user;
   }
   # nalezeni pripadne jiz existujiciho zaznamu
   my $select=q{
      select
         *
      from
         } . $$zmena{_table} . q{
   };
   my ($where,$privalues)=$pkg->where_zmena($zmena);
   $select.=q{
      where
         } . $where . q{
   };
   my $sth_sel=$dbh->prepare($select);
   $sth_sel->execute(@{$privalues});
   if($sth_sel->rows>0) {
      #--- zaznam BYL NALEZEN - udelam update - vhodne kvuli _add_ at opravdu prihazuji
      $pkg->update($zmena);
   }
   else {
      #--- zaznam NEBYL NALEZEN - udelam replace (ale mohl bych udelat i insert, ale pro jistotu, kdyby tam tesne predtim nekdo jiz zaznam vlozil (ovsem pravdepodobnost povazuji za mizivou) - sice to pak neni dobre, ale pri rade zapisu by zhrouceni byla katastrofa)
      # uplne ciste by to bylo kdybych opravdu vyvolal jen insert a kdyby to selhalo, tak bych znovu (vlastne rekurzivne) vyvolal $pkg->replace
      $pkg->insert($zmena,"replace");
   }
   $sth_sel->finish;
}

#- zapise metodou update zmeny uvedene v hashi %$zmena
#  tuto funkci uzivam jen jako pomocnou k replace - proste mimo tuto knihovnu ji zatim neuzivam
sub update {
   my ($pkg_p, $zmena)=@_;
   # zmena datzapis a uzivzapis, ktere predpokladam u kazde tabulky
   if(!defined($$zmena{datzapis}) and !$$zmena{_bezdatzapis}) { # jen kdyz neni definovano jiz v $zmena
      $$zmena{datzapis}=My->datetime_now;
      $$zmena{uzivzapis}=My->remote_user;
   }
   # definice dotazu, ktery meni udaje
   my $update=q{
      update
         } . $$zmena{_table} . q{
      set
   };
   my @values;
   foreach my $name (sort keys %{$zmena}) {
      if($name=~/^([^_].*)$/) {
         $update.= $1 . "=" . "?,";
         push(@values,$$zmena{$name});
      }
      elsif($name=~/^_add_(.+)$/) {
         $update.= $1 . "=" . $1 . "+" . "?,";
         push(@values,$$zmena{$name});
      }
   }
   chop($update); # odstraneni carky na konci
   my ($where,$privalues)=$pkg->where_zmena($zmena);
   $update.=q{
      where
         } . $where . q{
   };
   my $sth=$dbh->prepare($update);
   $sth->execute(@values,@{$privalues});
}

#- fce likviduje dle klicu uvedenych v hashi %$zmena zaznamy
# Write->delete({
#    _table => "Vyroba_evidence.Stavvyr",
#    _key_org => get_env("org"),
#    _key_zak => "XXXXXXXXXXXXXX",
#    _key_kmint => "XXX",
#    _key_str => "XXXX",
# });
sub delete {
   my ($pkg_p, $zmena)=@_;
   # definice dotazu, ktery maze zaznamy
   my $delete=q{
      delete from
         } . $$zmena{_table} . q{
   };
   my ($where,$privalues)=$pkg->where_zmena($zmena);
   $delete.=q{
      where
         } . $where . q{
   };
   my $sth=$dbh->prepare($delete);
   $sth->execute(@{$privalues});
}

#- pomocna funkce pro fce "replace" a "update" sestavuje podminku where z hashe %$zmena
sub where_zmena {
   my ($pkg_p, $zmena)=@_;
   my $where="";
   my $privalues;
   foreach my $prikey (grep(/^_key_/,keys %$zmena)) {
      push(@{$privalues}, $$zmena{$prikey});
      my $key=$prikey;
      $key=~s/^_key_//;
      $where.= $key . "=? and ";
   }
   $where= substr($where,0,length($where)-5); # zbaveni se " and " na konci retezce
   return ($where,$privalues);
}

#- vraci seznam sloupcu tabulky, fce je casto uzivana prave pri zapisu a proto je v tomto balicku
sub show_tablecols {
   my ($pkg_p, $table)=@_;
   my @cols;
   my $sth=$dbh->prepare(q{
      show
         columns
      from
         } . $table . q{
   });
   $sth->execute;
   while(my $ref=$sth->fetchrow_arrayref) {
      if($$ref[0] ne "datstamp") {
         push(@cols,$$ref[0]);
      }
   }
   $sth->finish;
   return @cols;
}

return 1;

